
	// based on code from https://github.com/toji/gl-matrix

	var newIdentityMatrix = function () {
		return [1, 0, 0, 1, 0, 0];
	};

	var multiplyMatrix = function (a, b) {
		var a0 = a[0], a1 = a[1], a2 = a[2], a3 = a[3], a4 = a[4], a5 = a[5],
			b0 = b[0], b1 = b[1], b2 = b[2], b3 = b[3], b4 = b[4], b5 = b[5];

		return [(a0 * b0 + a2 * b1),
				(a1 * b0 + a3 * b1),
				(a0 * b2 + a2 * b3),
				(a1 * b2 + a3 * b3),
				(a0 * b4 + a2 * b5 + a4),
				(a1 * b4 + a3 * b5 + a5)];
	};

	var rotateMatrix = function (a, rad) {
		var a0 = a[0], a1 = a[1], a2 = a[2], a3 = a[3], a4 = a[4], a5 = a[5],
			s = Math.sin(rad),
			c = Math.cos(rad);
		
		return [(a0 *  c + a2 * s),
				(a1 *  c + a3 * s),
				(a0 * -s + a2 * c),
				(a1 * -s + a3 * c),
				a4,
				a5];
	};

	var scaleMatrix = function (a, x, y) {
		var a0 = a[0], a1 = a[1], a2 = a[2], a3 = a[3], a4 = a[4], a5 = a[5];

		return [(a0 * x),
				(a1 * x),
				(a2 * y),
				(a3 * y),
				a4,
				a5];
	};

	var translateMatrix = function (a, x, y) {
		var a0 = a[0], a1 = a[1], a2 = a[2], a3 = a[3], a4 = a[4], a5 = a[5];

		return [a0,
				a1,
				a2,
				a3,
				(a0 * x + a2 * y + a4),
				(a1 * x + a3 * y + a5)];
	};

	var transformPointFromMatrix = function(a, x, y) {
		var a0 = a[0], a1 = a[1], a2 = a[2], a3 = a[3], a4 = a[4], a5 = a[5];

		return {x: (a0 * x + a2 * y + a4),
				y: (a1 * x + a3 * y + a5)};
	};

	var degreesToRadians = function (deg) {
		return Math.PI * (deg / 180.0);
	};

	var radiansToDegrees = function (rad) {
		return rad * (180.0 / Math.PI);
	};

	var invertMatrix = function(a) {
		var aa = a[0], ab = a[1], ac = a[2], ad = a[3], atx = a[4], aty = a[5];
		var det = aa * ad - ab * ac;
		if(!det) {
			return null;
		}
		det = 1.0 / det;
		
		return [(ad * det),
				(-ab * det),
				(-ac * det),
				(aa * det),
				((ac * aty - ad * atx) * det),
				((ab * atx - aa * aty) * det)];
	};

